// $Id: CBinaryTree.cpp,v 1.10 2001/01/29 21:58:25 chinaski Exp $

// ------------------------------------------------------------------- //
// Autores: Diego Chaparro Gonzlez - dchaparro@acm.org                //
//          Jos Pelegrn Lpez - josepl@acm.org                       //
// Creado: 22/12/00                                                    //
// Prctica de Inteligencia Artificial.                                //
// ------------------------------------------------------------------- //

#include "CBinaryTree.hpp"

// ------------------------------------------------------------------- //
// CONSTRUIRARBOL                                                      //
// ------------------------------------------------------------------- //
CBinaryTree CBinaryTree::ConstruirArbol(CNodoBT* n) const
{
  CBinaryTree nuevo_arbol;
  nuevo_arbol.arbol = n;
  return nuevo_arbol;
}

// ------------------------------------------------------------------- //
// CBINARYTREE                                                         //
// ------------------------------------------------------------------- //
CBinaryTree::CBinaryTree(void)
{
  arbol = 0;
}

// ------------------------------------------------------------------- //
// CBINARYTREE                                                         //
// ------------------------------------------------------------------- //
CBinaryTree::CBinaryTree(TipoElemento e)
{
  CNodoBT* nodo = new CNodoBT;
  nodo->raiz = e;
  nodo->izquierdo = 0;
  nodo->derecho = 0;

  arbol = nodo; 
}

// ------------------------------------------------------------------- //
// CBINARYTREE                                                         //
// ------------------------------------------------------------------- //
CBinaryTree::CBinaryTree(const CBinaryTree& a)
{
  if (a.arbol)
    {
      // Copia de la raz.
      CNodoBT* nodo = new CNodoBT;
      nodo->raiz = a.arbol->raiz;
      
      // Copia del rbol izquierdo.
      if (a.arbol->izquierdo)
	{
	  CBinaryTree izq(ConstruirArbol(a.arbol->izquierdo));
	  nodo->izquierdo = izq.arbol;
	}
      else 
	nodo->izquierdo = 0;

      // Copia del rbol derecho.
      if (a.arbol->derecho)
	{
	  CBinaryTree der(ConstruirArbol(a.arbol->derecho));
	  nodo->derecho = der.arbol;
	}
      else
	nodo->derecho = 0;

      arbol = nodo;
    }
  else
    arbol = 0;
}

// ------------------------------------------------------------------- //
// OPERATOR=                                                           //
// ------------------------------------------------------------------- //
CBinaryTree& CBinaryTree::operator= (const CBinaryTree& a)
{
  if (a.arbol)
    arbol = a.arbol;
  else
    arbol = 0;
  return *this;
}

// ------------------------------------------------------------------- //
// COPIAR                                                              //
// ------------------------------------------------------------------- //
CBinaryTree& CBinaryTree::Copiar (const CBinaryTree& a)
{
  if (a.arbol)
    {
      // Copia de la raz.
      CNodoBT* nodo = new CNodoBT;
      nodo->raiz = a.arbol->raiz;
      
      // Copia del rbol izquierdo.
      if (a.arbol->izquierdo)
	{
	  CBinaryTree izq(ConstruirArbol(a.arbol->izquierdo));
	  nodo->izquierdo = izq.arbol;
	}
      else 
	nodo->izquierdo = 0;

      // Copia de rbol derecho.
      if (a.arbol->derecho)
	{
	  CBinaryTree der(ConstruirArbol(a.arbol->derecho));
	  nodo->derecho = der.arbol;
	}
      else
	nodo->derecho = 0;

      arbol = nodo;
    }
  else
    arbol = 0;

  return *this;
}

// ------------------------------------------------------------------- //
// BORRARARBOL                                                         //
// ------------------------------------------------------------------- //
void CBinaryTree::BorrarArbol(CNodoBT* nodo, bool nodo_raiz)
{
  CNodoBT* actual;
  if (nodo_raiz)
    actual = arbol;
  else
    actual = nodo;

  if (actual)
    {
      // Borramos el rbol izquierdo.
      //if (actual->izquierdo)
	BorrarArbol(actual->izquierdo, false);
      // Borramos el rbol derecho.
      //if (actual->derecho)
	BorrarArbol(actual->derecho, false);
      // Borramos el rbol.
      delete actual;
      //actual = 0;
      cout<<"borrado"<<endl;
    }
}

// ------------------------------------------------------------------- //
// INSERTARRAIZ                                                        //
// ------------------------------------------------------------------- //
void CBinaryTree::InsertarRaiz(TipoElemento e)
{
  if (arbol) // El rbol ya existe.
    arbol->raiz = e;
  else // El rbol todava no existe.
    {
      CNodoBT *nodo = new CNodoBT;
      nodo->raiz = e;
      nodo->izquierdo = 0;
      nodo->derecho = 0;
      arbol = nodo;
    }
}

// ------------------------------------------------------------------- //
// INSERTARIZQUIERDA                                                   //
// ------------------------------------------------------------------- //
void CBinaryTree::InsertarIzquierda(TipoElemento e)
{
  if (arbol)
    {
      if (!arbol->izquierdo) // Existe un rbol izquierdo ya.
	{
	  CNodoBT *nodo = new CNodoBT;
	  nodo->izquierdo = 0;
	  nodo->raiz = e;
	  nodo->derecho = 0;
	  arbol->izquierdo = nodo;
	}
      else // Todava no existe el rbol izquierdo.
	arbol->izquierdo->raiz = e;
    }
  else throw PunteroNuloEx();
}

// ------------------------------------------------------------------- //
// INSERTARIZQUIERDA                                                   //
// ------------------------------------------------------------------- //
void CBinaryTree::InsertarIzquierda(const CBinaryTree& a)
{
  if (arbol)
    arbol->izquierdo = a.arbol;
  else throw PunteroNuloEx();
}

// ------------------------------------------------------------------- //
// INSERTARDERECHA                                                     //
// ------------------------------------------------------------------- //
void CBinaryTree::InsertarDerecha(TipoElemento e)
{
  if (arbol)
    {
      if (!arbol->derecho) // Existe ya un rbol derecho.
      	{
	  CNodoBT *nodo = new CNodoBT;
	  nodo->izquierdo = 0;
	  nodo->raiz = e;
	  nodo->derecho = 0;
	  arbol->derecho = nodo;
	}
      else // Todava no existe un rbol derecho.
	arbol->derecho->raiz = e;
    }
  else throw PunteroNuloEx();
}

// ------------------------------------------------------------------- //
// INSERTARDERECHA                                                     //
// ------------------------------------------------------------------- //
void CBinaryTree::InsertarDerecha(const CBinaryTree& a)
{
  if (arbol)
    arbol->derecho = a.arbol;
  else throw PunteroNuloEx();
}

// ------------------------------------------------------------------- //
// GRADO                                                               //
// ------------------------------------------------------------------- //
int CBinaryTree::Grado(void)
{
  int aux = 0;
  if (arbol)
    {
      int grado_izquierdo = (ConstruirArbol(arbol->izquierdo)).Grado();
      int grado_derecho = (ConstruirArbol(arbol->derecho)).Grado();
      aux = 1 + Max2(grado_izquierdo, grado_derecho);
    }

  return aux;
}
