// $Id: CBinaryTree.hpp,v 1.9 2001/01/29 21:58:25 chinaski Exp $

// ------------------------------------------------------------------- //
// Autores: Diego Chaparro Gonzlez - dchaparro@acm.org                //
//          Jos Pelegrn Lpez - josepl@acm.org                       //
// Creado: 22/12/00                                                    //
// Prctica de Inteligencia Artificial.                                //
// ------------------------------------------------------------------- //

#if !defined( _CBinaryTree_HPP_ )
#define _CBinaryTree_HPP_

#include "CExceptions.hpp"
#include <stdio.h>
#include <iostream.h>

typedef enum {UNO, N, SUMA, MULTIPLICACION, 
	      RESTA, DIVISION, POTENCIA} TipoElemento;

class CNodoBT
{
  friend class CBinaryTree;
private:
  TipoElemento raiz;
  CNodoBT* izquierdo;
  CNodoBT* derecho;  
};

inline int Max2(int x, int y)
{
  if (x > y)
    return x;
  else
    return y;
}

class CBinaryTree
{
private:
  CNodoBT* arbol;
  // ------------------------------------------------------------------- //
  // CONSTRUIRARBOL                                                      //
  // Precondiciones: --                                                  //
  // Postcondiciones: Construye un rbol a partir de un nodo.            //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  CBinaryTree ConstruirArbol(CNodoBT* n) const;
public:
  // ------------------------------------------------------------------- //
  // CBINARYTREE                                                         //
  // Precondiciones: --                                                  //
  // Postcondiciones: Construye un rbol vaco.                          //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  CBinaryTree(void);
  // ------------------------------------------------------------------- //
  // CBINARYTREE                                                         //
  // Precondiciones: --                                                  //
  // Postcondiciones: Construye un rbol con un elemento en la raz,     //
  //                  y sus rboles izquierdo y derecho vacos.          //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  CBinaryTree(TipoElemento e);
  // ------------------------------------------------------------------- //
  // CBINARYTREE                                                         //
  // Precondiciones: --                                                  //
  // Postcondiciones: Constructor de copia.                              //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  CBinaryTree(const CBinaryTree& a);
  // ------------------------------------------------------------------- //
  // OPERATOR=                                                           //
  // Precondiciones: --                                                  //
  // Postcondiciones: Operador de asignacin.                            //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  CBinaryTree& operator= (const CBinaryTree& a);
  // ------------------------------------------------------------------- //
  // COPIAR                                                              //
  // Precondiciones: --                                                  //
  // Postcondiciones: Devuelve un nuevo arbol copia del anterior.        //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  CBinaryTree& Copiar (const CBinaryTree& a);              
  // ------------------------------------------------------------------- //
  // BORRARARBOL                                                         //
  // Precondiciones: --                                                  //
  // Postcondiciones: Elimina la memoria ocupada por el rbol.           //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  void BorrarArbol(CNodoBT* nodo = 0, bool nodo_raiz = true);
  // ------------------------------------------------------------------- //
  // ESVACIO                                                             //
  // Precondiciones: --                                                  //
  // Postcondiciones: Determina si un rbol es vaco.                    //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  inline bool EsVacio(void) const 
    { return (arbol == 0); }
  // ------------------------------------------------------------------- //
  // INSERTARRAIZ                                                        //
  // Precondiciones: --                                                  //
  // Postcondiciones: Si el rbol es vaco, genera un nuevo rbol con    //
  //                  el elemento pasado. Si no es vaco, sustituye      //
  //                  el elemento actual por el nuevo.                   //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  void InsertarRaiz(TipoElemento e);
  // ------------------------------------------------------------------- //
  // INSERTARIZQUIERDA                                                   //
  // Precondiciones: El rbol no debe ser vaco.                         //
  // Postcondiciones: Si el rbol izquierdo es vaco, se genera un nuevo //
  //                  rbol, si no es vaco sustituye la raz del        //
  //                  izquierdo.                                         //
  // Excepciones: PunteroNuloEx                                          //
  // ------------------------------------------------------------------- //  
  void InsertarIzquierda(TipoElemento e);
  // ------------------------------------------------------------------- //
  // INSERTARIZQUIERDA                                                   //
  // Precondiciones: El rbol no debe ser vaco.                         //
  // Postcondiciones: El rbol pasado como parmetro pasa a ser el nuevo //
  //                  rbol izquierdo.                                   //
  // Excepciones: PunteroNuloEx                                          //
  // ------------------------------------------------------------------- //  
  void InsertarIzquierda(const CBinaryTree& a);
  // ------------------------------------------------------------------- //
  // INSERTARDERECHA                                                     //
  // Precondiciones: El rbol no debe ser vaco.                         //
  // Postcondiciones: Si el rbol derecho es vaco, se genera un nuevo   //
  //                  rbol, si no es vaco sustituye la raz del        // 
  //                  derecho                                            //
  // Excepciones: PunteroNuloEx                                          //
  // ------------------------------------------------------------------- //    
  void InsertarDerecha(TipoElemento e);
  // ------------------------------------------------------------------- //
  // INSERTARDERECHA                                                     //
  // Precondiciones: El rbol no debe ser vaco.                         //
  // Postcondiciones: Si el rbol derecho es vaco, se genera un nuevo   //
  //                  rbol, si no es vaco sustituye la raz del        // 
  //                  derecho                                            //
  // Excepciones: PunteroNuloEx                                          //
  // ------------------------------------------------------------------- // 
  void InsertarDerecha(const CBinaryTree& a);
  // ------------------------------------------------------------------- //
  // GRADO                                                               //
  // Precondiciones: --                                                  //
  // Postcondiciones: Obtiene el grado de un rbol.                      //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- // 
  int Grado(void);
  //void Copiar (CBinaryTree& Arbol);
  // ------------------------------------------------------------------- //
  // GETRAIZ                                                             //
  // Precondiciones: El rbol no debe ser vaco.                         //
  // Postcondiciones: Se devuelve la raz del rbol binario.             //
  // Excepciones: PunteroNuloEx                                          //
  // ------------------------------------------------------------------- // 
  inline TipoElemento GetRaiz(void) const 
    {  
      if (arbol)
	return arbol->raiz;
      else throw PunteroNuloEx();
    }
  // ------------------------------------------------------------------- //
  // GETARBOLIZQUIERDO                                                   //
  // Precondiciones: El rbol no debe ser vaco.                         //
  // Postcondiciones: Se devuelve una copia del rbol izquierdo.         //
  // Excepciones: PunteroNuloEx                                          //
  // ------------------------------------------------------------------- //
  inline CBinaryTree GetArbolIzquierdo(void) const
    {
      if (arbol){
	return ConstruirArbol(arbol->izquierdo);
      }
      else throw PunteroNuloEx();
    }
  // ------------------------------------------------------------------- //
  // GETARBOLDERECHO                                                     //
  // Precondiciones: El rbol no debe ser vaco.                         //
  // Postcondiciones: Se devuelve una copia del rbol derecho.           //
  // Excepciones: PunteroNuloEx                                          //
  // ------------------------------------------------------------------- //  
  inline CBinaryTree GetArbolDerecho(void) const
    {
      if (arbol){
	return ConstruirArbol(arbol->derecho);
      }
      else throw PunteroNuloEx(); 
    }
};

#endif // _CBinaryTree_HPP_
