// $Id: CTree.hpp,v 1.6 2001/01/29 18:20:59 chinaski Exp $

// ------------------------------------------------------------------- //
// Autores: Diego Chaparro Gonzlez - dchaparro@acm.org                //
//          Jos Pelegrn Lpez - josepl@acm.org                       //
// Creado: 22/12/00                                                    //
// Prctica de Inteligencia Artificial.                                //
// ------------------------------------------------------------------- //

#if !defined( _CTree_HPP_ )
#define _CTree_HPP_

#include "CBinaryTree.hpp"
#include "CExceptions.hpp"
#include <list>

class CTree;

class CNodoT
{
  friend class CTree;
private:
  CBinaryTree* raiz;
  list<CTree*> arboles;
};

class CTree
{
private:
  CNodoT* arbol;
public:
  // ------------------------------------------------------------------- //
  // CTREE                                                               //
  // Precondiciones: --                                                  //
  // Postcondiciones: Construye un rbol vaco.                          //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  CTree(void);
  // ------------------------------------------------------------------- //
  // CTREE                                                               //
  // Precondiciones: --                                                  //
  // Postcondiciones: Construye un rbol con un rbol binario en la raz //
  //                  y sin hijos.                                       //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  CTree(const CBinaryTree& bt);
  // ------------------------------------------------------------------- //
  // CTREE                                                               //
  // Precondiciones: --                                                  //
  // Postcondiciones: Construye un rbol con un rbol binario en la raz //
  //                  y con hijos.                                       //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  CTree(const CBinaryTree& bt,
        list<CTree*> as);
  // ------------------------------------------------------------------- //
  // ESVACIO                                                             //
  // Precondiciones: --                                                  //
  // Postcondiciones: Determina si un rbol es vaco.                    //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  inline bool EsVacio(void) const 
    { return ((arbol->arboles).size() == 0); }
  // ------------------------------------------------------------------- //
  // BORRARARBOL                                                         //
  // Precondiciones: --                                                  //
  // Postcondiciones: Elimina de la memoria el rbol.                    //
  // Excepciones: --                                                     //
  // ------------------------------------------------------------------- //
  void BorrarArbol(CNodoT* nodo = 0);
  // ------------------------------------------------------------------- //
  // INSERTAREXPRESION                                                   //
  // Precondiciones: El rbol debe ser no vaco.                         //
  // Postcondiciones: Inserta un rbol binario en la raz.               //
  // Excepciones: PunteroNuloEx                                          //
  // ------------------------------------------------------------------- //
  void InsertarExpresion(const CBinaryTree& a);
  // ------------------------------------------------------------------- //
  // INSERTARARBOL                                                       //
  // Precondiciones: El rbol debe ser no vaco.                         //
  // Postcondiciones: Crea un nuevo hijo del rbol cuya raz es el rbol //
  //                  binario dado.                                      //
  // Excepciones: PunteroNuloEx                                          //
  // ------------------------------------------------------------------- //
  void InsertarArbol(const CBinaryTree& a);
  // ------------------------------------------------------------------- //
  // GETRAIZ                                                             //
  // Precondiciones: El rbol debe ser no vaco.                         //
  // Postcondiciones: Devuelve la raz del rbol.                        //
  // Excepciones: PunteroNuloEx                                          //
  // ------------------------------------------------------------------- //
  inline CBinaryTree GetRaiz(void) const 
    {
      if (arbol)
	return CBinaryTree(*(arbol->raiz));
      else
	throw PunteroNuloEx();
    }
  // ------------------------------------------------------------------- //
  // GETARBOLES                                                          //
  // Precondiciones: El rbol debe ser no vaco.                         //
  // Postcondiciones: Devuelve los hijos del rbol.                      //
  // Excepciones: PunteroNuloEx                                          //
  // ------------------------------------------------------------------- //  
  inline list<CTree*> GetArboles(void) const
    {
      if (arbol)
	return arbol->arboles;
      else
	throw PunteroNuloEx();
    }
};

#endif
  
