// $Id: bni.cpp,v 1.10 2001/01/29 23:34:20 dchaparro Exp $

// ------------------------------------------------------------------- //
// Autores: Diego Chaparro Gonzlez - dchaparro@acm.org                //
//          Jos Pelegrn Lpez - josepl@acm.org                       //
// Creado: 22/12/00                                                    //
// Prctica de Inteligencia Artificial.                                //
// ------------------------------------------------------------------- //

// ------------------------------------------------------------------- //
// Parmetros de entrada al programa principal                         //
// 1 - Fichero que contiene secuencias, con el formato:                //
//          1 2 3 4 5                                                  //
//          1 2 4 8 16                                                 //
//          0 0 4 18 48                                                //
//          [...]                                                      //
//     Cada secuencia en una lnea. Se comprobarn todas las           //
//     secuencias hasta final de fichero.                              //
//     Por cada secuencia, se devolver la expresin que forma dicha   //
//     secuencia en el formato indicado en el encunciado:              //
//          Expresin 1: ...                                          //
//          Expresin 2: "pot + 1 1 n"                                //
//          Expresin 3: ...                                          //
//          [...]                                                      //
// ------------------------------------------------------------------- //

#include <vector>
#include <stdio.h>
#include "func.hpp"
#include "CBinaryTree.hpp"

#define PARAMETROS_ENTRADA_MAIN 2

// ------------------------------------------------------------------- //
// O B T E N E R S E C U E N C I A S                                   //
// Precondiciones: ??                                                  //
// Postcondiciones: ??
// Excepciones: ??                                                     //
// ------------------------------------------------------------------- //
vector< vector<int> > ObtenerSecuencias(const char* fich);

// ------------------------------------------------------------------- //
// M A I N                                                             //
// ------------------------------------------------------------------- //
int main(int argc, char *argv[])
{
  CBinaryTree ArbolBin;
  if (argc != PARAMETROS_ENTRADA_MAIN)
    {
      cout<<"Parmetros incorrectos."<<endl;
      cout<<"Uso: "<<argv[0]<<" <fichero_secuencias>"<<endl;
      return EXIT_FAILURE;
    }
  
  // Obtenemos las secuencias.
  vector< vector<int> > secuencias;
  secuencias = ObtenerSecuencias(argv[1]);
  

  // Buscamos por medio de la "Bsqueda de Profundizacin Iterativa",
  // la expresin que se ajusta a cada una de las secuencias.
  vector< vector<int> >::iterator actual = secuencias.begin();
  vector< vector<int> >::iterator final = secuencias.end();
  while (actual != final){
    ArbolBin = BusquedaPI (*actual);
    if (!( ArbolBin.EsVacio() )) {
      int i = 0;
      cout << "**************************************************"<< endl;
      cout << "* [ " <<(*actual)[i];
      while (i++ < (*actual).size()-1)
	cout << ", " << (*actual)[i];
      cout << "... ]: " << RepresentarExpr(ArbolBin) << endl;
      cout << "**************************************************"<< endl;
      actual++;
    }
  }
    
  return EXIT_SUCCESS;
}

// ------------------------------------------------------------------- //
// O B T E N E R S E C U E N C I A S                                   //
// ------------------------------------------------------------------- //
vector< vector<int> > ObtenerSecuencias(const char* fich)
{
  vector< vector<int> > secs; // Vector resultado.

  FILE *fsecuencias; // Fichero que contiene las secuencias.
  fsecuencias = fopen(fich, "r");

  if (fsecuencias)
    {      
      vector<int> secuencia; // Secuencia i.
      int num;               // Numero actual.
      char ch;               // Buscamos el caracter fn de lnea que
                             // determina el fin cada secuencia.
      while(fscanf(fsecuencias,"%d%c", &num, &ch) != EOF)
	{
	  secuencia.push_back(num);
	  if (ch == '\n') 
	    {
	      secs.push_back(secuencia);
	      secuencia.clear();
	    }
	}

      fclose(fsecuencias);
    }
  else
    cout<<fich<<" no existe"<<endl;

  return secs;
}

